import { FrontOfficeTicketData } from "./frontOfficeTicketData";

export class FrontOfficeClient {
    constructor(private readonly connector: string) {
        this.connector = connector;
    }

    /**
     * Creates JSON object with mandatory parameters representing body that can be send as POST to /tickets endpoint.
     * @return JSON object
     */
    getSingleTicketData(stateId: string, holderType: string, holderValue: string, template: string): FrontOfficeTicketData {
        return {
            type: "ticket",
            stateId: stateId,
            holder: {
                type: holderType,
                holder: holderValue,
            },
            documentData: {
                templateName: template,
            },
        };
    }

    /**
     * Creates Front Office ticket.
     * @param data JSON object that will be given as body
     * @returns Promise of ID of created ticket.
     */
    async createSingleTicket(data: FrontOfficeTicketData): Promise<string> {
        const json = await this.fetchFrontOffice("tickets", "POST", data);

        if (json.guid === undefined) {
            throw new Error("Could not parse ID of created ticket.");
        }

        return json.guid;
    }

    private async fetchFrontOffice(relativePath: string, method: string, data: FrontOfficeTicketData): Promise<any> {
        const url = this.connector + relativePath;

        const response = await fetch(url, {
            method: method,
            headers: {
                "Content-Type": "application/json",
            },
            body: JSON.stringify(data),
        });

        if (!response.ok) {
            throw new Error(`Web endpoint error: HTTP Code ${response.status} - '${await response.text()}'.`);
        }

        return await response.json();
    }
}
